/*
 *
 * exESTMLP.cpp  Estimates the size of the stochastic LP.
 *
 *         Reads problem from file <file>.spl and estimates the number of
 *         rows, columns and the elements.
 *         (compile with /DOSLMSDLL on INTEL platforms.)
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "scn_c_api.h"

#define MXSCN 1000            /* maximum scenarios */

int main( long argc, char *argv[] ) {

/* Initialize EKKContext and EKKStoch objects */
    EKKContext *env=ekks_initializeContext();
    EKKStoch *stoch=ekks_newStoch(env,NULL,MXSCN);
	
/* 
   Read SPL file (if it exists from previous runs)
   or SMPS files
*/
	FILE *fp = fopen("prod_mix.spl","r");

    if (NULL==fp)
	{
		long type=ekks_readSMPSData(stoch,"prod_mix.core","prod_mix.time","prod_mix.stoch");                  
		if (type<0) exit(999);
		ekks_writeNativeData(stoch,"prod_mix.spl");  
	}
	else
	{	
		fclose(fp);
		ekks_readNativeData(stoch,"prod_mix.spl");
	}

/* Estimate the size of the stochastic LP */
	int nsize = ekks_estimateLPSize(stoch);
    printf("\n MYMSG: size of SLP is %d \n",nsize);                      

/* Write stochastic data out to files in SMPS format */
    ekks_outMatrixSMPS(stoch,SCENARIOS,1,"myapp.core","myapp.time","myapp.stoch");

    ekks_deleteStoch(stoch);                         
    ekks_endContext(env);

    printf("\n ************************************");
    printf("\n ***   All testing successful !   ***");
    printf("\n ************************************\n");fflush(stdout);
    
    return 0;
}
