/*
 * exNested.cpp:  Main driver for reading SPL-files and solving.
 *
 *         Reads problem from SPL file and solves using
 *         ekkse_nestedLSolve().
 *         (compile with  /DOSLMSDLL on INTEL platforms.)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>
#include "scn_c_api.h"

#define MXSCN 15000            /* maximum scenarios */

int main( long argc, char *argv[] ) {

/* Initialize EKKContext and EKKStoch objects */
    EKKContext *env=ekks_initializeContext();
    EKKStoch *stoch=ekks_newStoch(env,NULL,MXSCN);
	
/* 
   Read SPL file (if it exists from previous runs)
   or SMPS files
*/
	FILE *fp = fopen("prod_mix.spl","r");

    if (NULL==fp)
	{
		long type=ekks_readSMPSData(stoch,"prod_mix.core","prod_mix.time","prod_mix.stoch");                  
		if (type<0) exit(999);
		ekks_writeNativeData(stoch,"prod_mix.spl");  
	}
	else
	{	
		fclose(fp);
		ekks_readNativeData(stoch,"prod_mix.spl");
	}

/* Describe Full model and don't generate deterministic equivalent */
	ekks_describeFullModel(stoch,0);  

/* Set decomposition solver parameters */
    ekks_setMaxsubmodels(stoch,10);                      
	ekks_setScaleOn(stoch);
	ekks_setCrash(stoch,3);
	ekks_setPresolve(stoch,3);
	ekks_setFinalSimplexSolverOff(stoch);

/* Solve using nested decomposition with 
	minimum number of rows for each subproblem */
	ekks_setMinNumRows(stoch,500);
    ekkse_nestedLSolve(stoch,CUTBYROWSIZE,0);  
    
/* Check prod_mix optimal value */
	assert(fabs(ekks_getRobjvalue(stoch)+17730.32)<1.0e-6*17730.32);

/* Print objective distribution */
	ekks_printObjectiveDistribution(stoch);

/* Delete all objects */
    ekks_deleteStoch(stoch);
    ekks_endContext(env);

    printf("\n ************************************");
    printf("\n ***   All testing successful !   ***");
    printf("\n ************************************\n");fflush(stdout);
    
    return 0;
}
